# RegistrationApp for Django: Register users, with e-mail verification and CAPTCHA.
#
# Copyright (C) 2008  Juergen Brendel
#
# Dual licensed under AGPL and BSD license.

# Specifies the models (DB tables) that we need for the
# RegistrationApp.

import random
import datetime
import time

from django.contrib.auth.models import User
from django.db.models import Q
from django.db import models

from reg_settings import *

import captcha


class UserProfile(models.Model):
    # We store some additional information for each user.
    user = models.ForeignKey(User, unique=True)

    # Some additional fields and flags
    confirmation_time = models.DateTimeField()
    is_confirmed      = models.IntegerField()

    def __str__(self):
        return "user-id: %s   confirmation_time: %s   is_confirmed: %s" % (self.user.id, self.confirmation_time, self.is_confirmed)


class EmailConfirmation(models.Model):
    # We need to store information about the e-mail notification
    # in a database table, so that we know the confirmation link
    # when the user clicks on it.
    user           = models.ForeignKey(User, unique=True)
    key            = models.CharField(maxlength=100)
    user_name      = models.CharField(maxlength=100, db_index=True)
    date_created   = models.DateTimeField()
    date_confirmed = models.DateTimeField(null=True)
    is_used_up     = models.IntegerField(db_index=True)

    def __str__(self):
        return "User ID: %s    Username: %s    Key: %s    Date: %s" % (user, user_name, key, date)

    def used_up(self):
        self.date_confirmed = datetime.datetime.now()
        self.is_used_up     = 1
        self.save()

    @classmethod
    def factory(cls, user):
        key_val       = str(random.randint(100000000000000000000000L, 1000000000000000000000000L))
        key_val      += ''.join(str(time.time()).split("."))
        date_val      = datetime.datetime.now()
        user_name_val = user.username
        user_id_val   = user.id
        user.email_user(email_sub_confirmation, email_msg_confirmation % key_val, site_email)
        ec = EmailConfirmation.objects.create(key=key_val, user_name=user_name_val, user_id=user_id_val, date_created=date_val, is_used_up=0)
        ec.save()
        return ec


class Captchas(models.Model):
    # To prevent replay attacks on our CAPTCHA, we don't use fancy hashing
    # or anything like that in an input form's hidden field. Instead, we
    # store the value in a DB table, along with a randomly generated key.
    # That key is placed as a hidden parameter into the registration form.
    # Then, when the form values arrive, we can look up the actual character
    # string for the CAPTCHA in this table here.
    key          = models.CharField(maxlength=100, db_index=True)
    value        = models.CharField(maxlength=10)
    date_created = models.DateTimeField()

    @classmethod
    def factory(cls):
        key_val   = str(random.randint(100000000000000000000000L, 1000000000000000000000000L))
        key_val  += ''.join(str(time.time()).split("."))
        date_val  = datetime.datetime.now()
        ca = Captchas(key=key_val, value=captcha.make_captcha_string(), date_created=date_val)
        ca.save()
        return ca


